create or replace PROCEDURE WOANCESTORBUILD (Level_In IN number, WoNum_In IN varchar2, Parent_In IN varchar2, orgid_in IN varchar2, siteID_in IN varchar2) AS 
--
-- The procedure is called recursively to build the woancestor table
-- in maximo. Since it does a complete rebuild of the user's entries it assumes the table
-- has been previously truncated, or all the user's heirarchy of records deleted.
-- Note Maximo should also be down, or idle, ie no users so it does not interfere 
-- with the process.
--
New_WoNum   varchar2(16);
New_Parent varchar2(16);
New_SiteId varchar2(8);
New_OrgId varchar2(8);
Parent_Check integer := 0;

BEGIN
  If Level_In = null or WoNum_in = null then
    return;
  else
    --
    -- Insert woancestor record for root.
    -- Find workorder record for parent if it exists.
    -- Does this new workorder have a parent
    -- If so, call WoAncestorBuild again with new values.
    -- otherwise return.
    --
    If Level_In = 0 then
      Insert into woancestor
        (ancestor, wonum ,hierarchylevels, orgid, siteid, woancestorid)
        Values
        (WoNum_In, WoNum_In, 0, Orgid_in, SiteId_in, woancestorseq.nextval);
    end if;
      
    If Parent_In is not null then
      --
      -- Make sure parent exists before proceeding.
      --
      Select count(*) into Parent_Check from workorder where WoNum = Parent_in;
      --
      If Parent_Check > 0 then
        Insert into woancestor
          (ancestor, wonum ,hierarchylevels, orgid, siteid, woancestorid)
          Values
          (Parent_In, WoNum_In, Level_In+1, orgid_in, siteid_in, woancestorseq.nextval);
        Select WoNum, parent, orgid, siteid into new_wonum, new_parent, new_orgid, new_siteid
        from WORKORDER
        where WoNum = Parent_in;
        If new_parent is null then
          return;
        else
          WoAncestorBuild(Level_In+1,WoNum_In, new_parent, New_orgid, New_siteid);
        end if;
      else
        null;
        -- No matching work order for parent reference.
        -- dbms_output.put_line('WorkOrder: '||WoNum_In||' parent reference is invalid, parent ref: '||Parent_In);
      end if;
    end if;
  end if;
END WOANCESTORBUILD;